@@ -21,19 +21,19 @@ cond_5: {
         }
     }
     expect: {
-        some_condition() && some_other_condition() ? do_something() : alternate();
+        (some_condition() && some_other_condition() ? do_something : alternate)();
         if (some_condition() && some_other_condition()) do_something();
     }
 }
 
 dead_code_const_annotation_regex: {
     options = {
-        booleans     : true,
-        conditionals : true,
-        dead_code    : true,
-        evaluate     : true,
-        expression   : true,
-        loops        : true,
+        booleans: true,
+        conditionals: true,
+        dead_code: true,
+        evaluate: true,
+        expression: true,
+        loops: true,
     }
     input: {
         var unused;
@@ -82,8 +82,9 @@ drop_value: {
 
 wrongly_optimized: {
     options = {
-        conditionals: true,
         booleans: true,
+        conditionals: true,
+        dead_code: true,
         evaluate: true,
         expression: true,
     }
@@ -99,8 +100,8 @@ wrongly_optimized: {
         function func() {
             foo();
         }
-        // TODO: optimize to `func(), bar()`
-        if (func(), 1) bar();
+        func(), 1;
+        bar();
     }
 }
 
@@ -195,7 +196,7 @@ negate_iife_5_off: {
         expression: true,
         negate_iife: false,
         sequences: true,
-    };
+    }
     input: {
         if ((function(){ return t })()) {
             foo(true);
@@ -248,7 +249,9 @@ issue_1254_negate_iife_nested: {
 conditional: {
     options = {
         expression: true,
-        pure_funcs: [ "pure" ],
+        pure_funcs: [
+            "pure"
+        ],
         side_effects: true,
     }
     input: {
@@ -314,7 +317,35 @@ iife: {
         typeof function g() {}();
     }
     expect: {
-        x = 42, function a() {}(), function b() {}(), function c() {}(),
-        function d() {}(), function e() {}(), function f() {}(), typeof function g() {}();
+        x = 42,
+        function a() {}(),
+        !function b() {}(),
+        ~function c() {}(),
+        +function d() {}(),
+        -function e() {}(),
+        void function f() {}(),
+        typeof function g() {}();
+    }
+}
+
+iife_drop_side_effect_free: {
+    options = {
+        expression: true,
+        sequences: true,
+        side_effects: true,
+    }
+    input: {
+        x = 42;
+        (function a() {})();
+        !function b() {}();
+        ~function c() {}();
+        +function d() {}();
+        -function e() {}();
+        void function f() {}();
+        typeof function g() {}();
+    }
+    expect: {
+        x = 42,
+        typeof void 0;
     }
 }
